@@ -6,10 +6,11 @@ Process Pools.
 import multiprocessing
 import itertools
 import threading
-import uuid
 
+from multiprocessing.pool import Pool, worker
 from multiprocessing.pool import RUN as POOL_STATE_RUN
 from celery.datastructures import ExceptionInfo
+from celery.utils import gen_unique_id
 from functools import partial as curry
 
 
@@ -47,7 +48,7 @@ class TaskPool(object):
 
         """
         self._processes = {}
-        self._pool = multiprocessing.Pool(processes=self.limit)
+        self._pool = Pool(processes=self.limit)
 
     def stop(self):
         """Terminate the pool."""
@@ -68,7 +69,7 @@ class TaskPool(object):
         callbacks = callbacks or []
         errbacks = errbacks or []
         meta = meta or {}
-        tid = str(uuid.uuid4())
+        tid = gen_unique_id()
 
         self._processed_total = self._process_counter.next()
 
